@@ -1,30 +1,23 @@
-#
-# Maximum Parsimony
-#
-sankoff.quartet <- function(dat, cost, p, l, weight) {
-  erg <- .Call('sankoffQuartet', sdat = dat, sn = p, scost = cost, sk = l)
-  sum(weight * erg)
-}
-
-
 #' Parsimony tree.
 #'
+#' \code{pratchet} implements the parsimony ratchet (Nixon, 1999) and is the
+#' preferred way to search for the best parsimony tree. For small number of taxa
+#' the function \code{\link{bab}} can be used to compute all most parsimonious
+#' trees.
 #'
 #' \code{parsimony} returns the parsimony score of a tree using either the
-#' sankoff or the fitch algorithm. \code{optim.parsimony} tries to find the
-#' maximum parsimony tree using either Nearest Neighbor Interchange (NNI)
-#' rearrangements or sub tree pruning and regrafting (SPR). \code{pratchet}
-#' implements the parsimony ratchet (Nixon, 1999) and is the preferred way to
-#' search for the best tree.  \code{random.addition} can be used to produce
-#' starting trees.
+#' sankoff or the fitch algorithm.
+#' \code{optim.parsimony} optimizes the topology using either Nearest Neighbor
+#' Interchange (NNI) rearrangements or sub tree pruning and regrafting (SPR) and
+#' is used inside \code{pratchet}. \code{random.addition} can be used to produce
+#' starting trees and is an option for the argument perturbation in
+#' \code{pratchet}.
 #'
 #' The "SPR" rearrangements are so far only available for the "fitch" method,
 #' "sankoff" only uses "NNI". The "fitch" algorithm only works correct for
 #' binary trees.
 #'
 #' @aliases parsimony
-#' @aliases optim.parsimony sankoff fitch pratchet
-#' random.addition acctran
 #' @param data A object of class phyDat containing sequences.
 #' @param tree tree to start the nni search from.
 #' @param method one of 'fitch' or 'sankoff'.
@@ -117,7 +110,7 @@ compressSites <- function(data) {
   ntaxa <- nrow(data)
   res <- vector("list", ntaxa)
   for(i in seq_len(ntaxa)) res[[i]] <- data[i, pos]
-  attrData$weight <- tapply(attrData$weight, index, sum)
+  attrData$weight <- as.vector(tapply(attrData$weight, index, sum))
   attrData$index <- NULL
   attrData$nr <- length(attrData$weight)
   attrData$compressed <- TRUE
@@ -200,21 +193,39 @@ upperBound <- function(x, cost = NULL) {
 #' \deqn{RI = \frac{MaxChanges - ObsChanges}{MaxChanges - MinChanges}}{RI = (MaxChanges - ObsChanges) / (MaxChanges - MinChanges)}
 #'
 #' @param data A object of class phyDat containing sequences.
-#' @param tree tree to start the nni search from.
+#' @param tree a phylogenetic tree, i.e. an object of class \code{phylo}.
 #' @param cost A cost matrix for the transitions between two states.
 #' @param sitewise return CI/RI for alignment or sitewise
-#'
+#' @returns a scalar or vector with the CI/RI vector.
 #' @seealso \code{\link{parsimony}}, \code{\link{pratchet}},
 #' \code{\link{fitch}}, \code{\link{sankoff}}, \code{\link{bab}},
 #' \code{\link{ancestral.pars}}
+#' @examples
+#' example(as.phylo.formula)
+#' lab <- tr$tip.label
+#' lab[79] <- "Herpestes fuscus"
+#' tr$tip.label <- abbreviateGenus(lab)
+#' X <- matrix(0, 112, 3, dimnames = list(tr$tip.label, c("Canis", "Panthera",
+#'             "Canis_Panthera")))
+#' desc_canis <- Descendants(tr, "Canis")[[1]]
+#' desc_panthera <- Descendants(tr, "Panthera")[[1]]
+#' X[desc_canis, c(1,3)] <- 1
+#' X[desc_panthera, c(2,3)] <- 1
+#' col <- rep("black", 112)
+#' col[desc_panthera] <- "red"
+#' col[desc_canis] <- "blue"
+#' X <- phyDat(X, "USER", levels=c(0,1))
+#' plot(tr, "f", tip.color=col)
+#' # The first two sites are homoplase free!
+#' CI(tr, X, sitewise=TRUE)
+#' RI(tr, X, sitewise=TRUE)
 #'
 #' @rdname CI
 #' @export
 CI <- function(tree, data, cost = NULL, sitewise = FALSE) {
-  if (sitewise) pscore <- sankoff(tree, data, cost = cost, site = "site")
-  else pscore <- sankoff(tree, data, cost = cost)
-  weight <- attr(data, "weight")
   data <- subset(data, tree$tip.label)
+  pscore <- sankoff(tree, data, cost, ifelse(sitewise, "site", "pscore"))
+  weight <- attr(data, "weight")
   m <- lowerBound(data, cost = cost)
   if (sitewise) {
     return( (m / pscore)[attr(data, "index")])
@@ -226,9 +237,8 @@ CI <- function(tree, data, cost = NULL, sitewise = FALSE) {
 #' @rdname CI
 #' @export
 RI <- function(tree, data, cost = NULL, sitewise = FALSE) {
-  if (sitewise) pscore <- sankoff(tree, data, cost = cost, site = "site")
-  else pscore <- sankoff(tree, data, cost = cost)
   data <- subset(data, tree$tip.label)
+  pscore <- sankoff(tree, data, cost, ifelse(sitewise, "site", "pscore"))
   weight <- attr(data, "weight")
   m <- lowerBound(data, cost = cost)
   g <- upperBound(data, cost = cost)
@@ -242,12 +252,6 @@ RI <- function(tree, data, cost = NULL, sitewise = FALSE) {
 }
 
 
-
-#
-# Sankoff
-#
-
-
 old2new.phyDat <- function(obj) {
   att <- attributes(obj)
   l <- length(obj)
@@ -274,59 +278,6 @@ new2old.phyDat <- function(data) {
 }
 
 
-prepareDataSankoff <- function(data) {
-  contrast <- attr(data, "contrast")
-  contrast[contrast == 0] <- 1e+06
-  contrast[contrast == 1] <- 0
-  for (i in seq_along(data)) data[[i]] <- contrast[data[[i]], , drop = FALSE]
-  data
-}
-
-
-fit.sankoff <- function(tree, data, cost,
-                        returnData = c("pscore", "site", "data")) {
-  tree <- reorder(tree, "postorder")
-  returnData <- match.arg(returnData)
-  node <- tree$edge[, 1]
-  edge <- tree$edge[, 2]
-  weight <- attr(data, "weight")
-  nr <- attr(data, "nr")
-  q <- length(tree$tip.label)
-  nc <- attr(data, "nc")
-  m <- length(edge) + 1
-  dat <- vector(mode = "list", length = m)
-  dat[1:q] <- data[tree$tip.label]
-  node <- as.integer(node - 1)
-  edge <- as.integer(edge - 1)
-  mNodes <- as.integer(max(node) + 1)
-  tips <- as.integer( (seq_along(tree$tip.label)) - 1)
-  res <- .Call('sankoff3', dat, as.numeric(cost), as.integer(nr),
-    as.integer(nc), node, edge, mNodes, tips)
-  root <- getRoot(tree)
-  erg <- .Call('C_rowMin', res[[root]], as.integer(nr), as.integer(nc))
-  if (returnData == "site") return(erg)
-  pscore <- sum(weight * erg)
-  result <- pscore
-  if (returnData == "data") {
-    result <- list(pscore = pscore, dat = res)
-  }
-  result
-}
-
-
-pnodes <- function(tree, data, cost) {
-  tree <- reorder(tree, "postorder")
-  node <- tree$edge[, 1]
-  edge <- tree$edge[, 2]
-  nr <- nrow(data[[1]])
-  nc <- ncol(data[[1]])
-  node <- as.integer(node - 1)
-  edge <- as.integer(edge - 1)
-  .Call('pNodes', data, as.numeric(cost), as.integer(nr), as.integer(nc),
-    node, edge)
-}
-
-
 indexNNI <- function(tree) {
   parent <- tree$edge[, 1]
   child <- tree$edge[, 2]
@@ -339,7 +290,7 @@ indexNNI <- function(tree) {
   pvector[child] <- parent
   tips  <- !logical(max(parent))
   tips[parent] <-  FALSE
-  # wahrscheinlich schneller: cvector <- allCildren(tree)
+  # wahrscheinlich schneller: cvector <- allChildren(tree)
   cvector <- vector("list", max(parent))
   for (i in seq_along(parent)) cvector[[parent[i]]] <- c(cvector[[parent[i]]],
       child[i])
@@ -360,64 +311,6 @@ indexNNI <- function(tree) {
 }
 
 
-sankoff.nni <- function(tree, data, cost, ...) {
-  if (is.rooted(tree)) tree <- reorder(unroot(tree), "postorder")
-  INDEX <-  indexNNI(tree)
-  rootEdges <- attr(INDEX, "root")
-  if (!inherits(data, "phyDat"))
-    stop("data must be of class phyDat")
-  levels <- attr(data, "levels")
-  l <- length(levels)
-  weight <- attr(data, "weight")
-  p <- attr(data, "nr")
-  i <- 1
-  tmp <- fit.sankoff(tree, data, cost, returnData = "data")
-  p0 <- tmp[[1]]
-  datf <- tmp[[2]]
-  datp <- pnodes(tree, datf, cost)
-
-  parent <- tree$edge[, 1]
-  m <- dim(INDEX)[1]
-  k <- min(parent)
-  pscore <- numeric(2 * m)
-
-  for (i in 1:m) {
-    ei <- INDEX[i, ]
-    datn <- datf[ei[1:4]]
-    if (!(ei[5] %in% rootEdges)) datn[1] <- datp[ei[1]]
-    pscore[(2 * i) - 1] <- sankoff.quartet(datn[ c(1, 3, 2, 4)],
-      cost, p, l, weight)
-    pscore[(2 * i)] <- sankoff.quartet(datn[ c(1, 4, 3, 2)],
-      cost, p, l, weight)
-  }
-  swap <- 0
-  candidates <- pscore < p0
-  while (any(candidates)) {
-
-    ind <- which.min(pscore)
-    pscore[ind] <- Inf
-    if (ind %% 2) swap.edge <- c(2, 3)
-    else swap.edge <- c(2, 4)
-
-    tree2 <- changeEdge(tree, INDEX[(ind + 1) %/% 2, swap.edge])
-    test <- fit.sankoff(tree2, data, cost, "pscore")
-
-    if (test >= p0) candidates[ind] <- FALSE
-    if (test < p0) {
-      p0 <- test
-      swap <- swap + 1
-      tree <- tree2
-      candidates[ind] <- FALSE
-      indi <- which(rep(colSums(apply(INDEX, 1, match, INDEX[(ind + 1) %/% 2, ],
-        nomatch = 0)) > 0, each = 2))
-      candidates[indi] <- FALSE
-      pscore[indi] <- Inf
-    }
-  }
-  list(tree = tree, pscore = p0, swap = swap)
-}
-
-
 #' @rdname parsimony
 #' @export
 optim.parsimony <- function(tree, data, method = "fitch", cost = NULL,
@@ -430,16 +323,17 @@ optim.parsimony <- function(tree, data, method = "fitch", cost = NULL,
 }
 
 
-
-
 #' @rdname parsimony
 #' @export
 pratchet <- function(data, start = NULL, method = "fitch", maxit = 1000,
                      minit = 100, k = 10, trace = 1, all = FALSE,
                      rearrangements = "SPR", perturbation = "ratchet", ...) {
+  if(inherits(data, "DNAbin") || inherits(data, "AAbin"))
+    data <- as.phyDat(data)
+  printevery <- 10L
   eps <- 1e-08
   trace <- trace - 1
-
+  ref <- names(data)
   start_trees <- vector("list", maxit)
   search_trees <- vector("list", maxit)
   tree <- NULL
@@ -468,13 +362,13 @@ pratchet <- function(data, start = NULL, method = "fitch", maxit = 1000,
     else tree <- stree(nTips, tip.label = nam)
     if(add_taxa) tree <- addTaxa(tree, attr(data, "duplicated"))
     if(!ROOTED) tree <- unroot(tree)
+    tree <- relabel(tree, ref)
     return(tree)
   }
 
-  if (perturbation != "random_addition"){
-    if(is.null(start)) start <- optim.parsimony(fastme.ols(dist.hamming(data)),
-                                       data, trace = trace-1, method = method,
-                                       rearrangements = rearrangements, ...)
+    if(is.null(start)) start <- optim.parsimony(random.addition(data),
+                                   data, trace = trace-1, method = method,
+                                   rearrangements = rearrangements, ...)
     tree <- start
     label <- intersect(tree$tip.label, names(data))
     if (!is.binary(tree)){
@@ -486,8 +380,7 @@ pratchet <- function(data, start = NULL, method = "fitch", maxit = 1000,
     attr(tree, "pscore") <- parsimony(tree, data, method = method, ...)
     mp <- attr(tree, "pscore")
     if (trace >= 0)
-      print(paste("Best pscore so far:", attr(tree, "pscore")))
-  }
+      cat("Parsimony score of initial tree:", attr(tree, "pscore"), "\n")
   FUN <- function(data, tree, method, rearrangements, ...)
     optim.parsimony(tree, data = data, method = method,
                     rearrangements = rearrangements, ...)
@@ -495,11 +388,11 @@ pratchet <- function(data, start = NULL, method = "fitch", maxit = 1000,
   if(!is.null(attr(data, "duplicated"))){
     result <- addTaxa(result, attr(data, "duplicated"))
   }
+  result <- relabel(result, ref)
+#  if (trace > 1) cat("optimize topology (NNI): ", pscore, "-->", psc, "\n")
+  hr <- hash(result)
   on.exit({
     if (!all && inherits(result, "multiPhylo")) result <- result[[1]]
-#    if(!is.null(attr(data, "duplicated")))
-#      result <- addTaxa(result, attr(data, "duplicated"))
-    #    else class(result) <- "multiPhylo"
     if (length(result) == 1) result <- result[[1]]
     env <- new.env()
     start_trees <- start_trees[seq_len(i)]
@@ -528,26 +421,24 @@ pratchet <- function(data, start = NULL, method = "fitch", maxit = 1000,
       bs_ind <- which(bsw > 0)
       bs_data <- getRows(data, bs_ind)
       attr(bs_data, "weight") <- bsw[bs_ind]
-      if(length(bs_ind) > 0)p_trees <- optim.parsimony(tree, bs_data,
+      if(length(bs_ind) > 0){
+        # p_trees <- random.addition(bs_data)  # 3 * ??
+        p_trees <- optim.parsimony(tree, bs_data,
           trace = trace, method = method, rearrangements = rearrangements, ...)
+      }
       else p_trees <- stree(length(data), tip.label = names(data))
-      trees <- optim.parsimony(p_trees, data, trace = trace,
-                     method = method, rearrangements = rearrangements, ...)
-    }
-    if (perturbation == "stochastic") {
-      p_trees <- rNNI(tree, floor(nTips / 2))
-      trees <- optim.parsimony(p_trees, data, trace = trace, method = method,
-                               rearrangements = rearrangements, ...)
-    }
-    if (perturbation == "random_addition") {
-      p_trees <- random.addition(data)
-      trees <- optim.parsimony(p_trees, data, trace = trace, method = method,
-                               rearrangements = rearrangements, ...)
     }
+    if (perturbation == "stochastic") p_trees <- rNNI(tree, floor(nTips / 2))
+    if (perturbation == "random_addition") p_trees <- random.addition(data)
+    trees <- optim.parsimony(p_trees, data, trace = trace, method = method,
+                             rearrangements = rearrangements, ...)
+    curr_tree <- trees
     if(!is.null(attr(data, "duplicated"))){
       p_trees <- addTaxa(p_trees, attr(data, "duplicated"))
       trees <- addTaxa(trees, attr(data, "duplicated"))
     }
+    trees <- relabel(trees, ref)
+    p_trees <- relabel(p_trees, ref)
     start_trees[[i]] <- p_trees
     search_trees[[i]] <- trees
     pscores <- attr(trees, "pscore")
@@ -555,68 +446,23 @@ pratchet <- function(data, start = NULL, method = "fitch", maxit = 1000,
     if ( (mp1 + eps) < mp) {
       kmax <- 1
       result <- trees
-      tree <- trees
+      tree <- curr_tree
+      hr <- hash(trees)
       mp <- mp1
     }
     else{
       kmax <- kmax + 1
-      if( all && (mp1 < (mp + eps)) && all(RF.dist(trees, result) > 0))
-        result <- c(result, trees)
+      if( all && (mp1 < (mp + eps))){ # && all(RF.dist(trees, result) > 0)
+        ht <- hash(trees)
+        if(!(ht %in% hr)){
+          hr <- c(hr, ht)
+          result <- c(result, trees)
+        }
+      }
     }
-    if (trace >= 0)
-      print(paste("Best pscore so far:", mp))
+    if (trace >= 0 &&  (!i%%printevery))
+      cat("\rIteration: ", i, ". Best parsimony score so far: ", mp, sep="")
     if ( (kmax >= k) && (i >= minit)) break()
   } # for
+  if (trace >= 0)cat("\n")
 }  # pratchet
-
-
-optim.sankoff <- function(tree, data, cost = NULL, trace = 1, ...) {
-  if (!inherits(tree, "phylo")) stop("tree must be of class phylo")
-  if (is.rooted(tree)) tree <- unroot(tree)
-  tree <- reorder(tree, "postorder")
-  if (!inherits(data, "phyDat")) stop("data must be of class phyDat")
-  addTaxa <- FALSE
-  mapping <- map_duplicates(data)
-  if (!is.null(mapping)) {
-    addTaxa <- TRUE
-    tree2 <- drop.tip(tree, mapping[, 1])
-    tree2 <- unroot(tree2)
-    tree <- reorder(tree2, "postorder")
-  }
-
-  rt <- FALSE
-  dat <- prepareDataSankoff(data)
-  l <- attr(dat, "nc")
-  if (is.null(cost)) {
-    cost <- matrix(1, l, l)
-    cost <- cost - diag(l)
-  }
-
-
-  tree$edge.length <- NULL
-  swap <- 0
-  iter <- TRUE
-  pscore <- fit.sankoff(tree, dat, cost, "pscore")
-
-  on.exit({
-    if (rt) tree <- acctran(tree, data)
-    if (addTaxa) {
-      if (rt) tree <- add.tips(tree, tips = mapping[, 1], where = mapping[, 2],
-          edge.length = rep(0, nrow(mapping)))
-      else tree <- add.tips(tree, tips = mapping[, 1], where = mapping[, 2])
-    }
-    attr(tree, "pscore") <- pscore
-    return(tree)
-  })
-
-  while (iter) {
-    res <- sankoff.nni(tree, dat, cost, ...)
-    tree <- res$tree
-    if (trace > 1) cat("optimize topology: ", pscore, "-->", res$pscore, "\n")
-    pscore <- res$pscore
-    swap <- swap + res$swap
-    if (res$swap == 0) iter <- FALSE
-  }
-  if (trace > 0) cat("Final p-score", pscore, "after ", swap,
-                     "nni operations \n")
-}
