@@ -1,7 +1,7 @@
 /*
  * ml.c
  *
- * (c) 2008-2021  Klaus Schliep (klaus.schliep@gmail.com)
+ * (c) 2008-2024  Klaus Schliep (klaus.schliep@gmail.com)
  *
  *
  * This code may be distributed under the GNU GPL
@@ -86,8 +86,11 @@ void scaleMatrix(double *X, int *nr, int *nc, int *result){
     double tmp;
     for(i = 0; i < *nr; i++) {
         tmp = 0.0;
-        for(j = 0; j < *nc; j++) tmp += X[i + j* *nr];
-        while(tmp < ScaleEPS){
+        for(j = 0; j < *nc; j++){
+//          if(X[i + j* *nr] < 0.0) X[i + j* *nr] = 0.0;
+          tmp += X[i + j* *nr];
+        }
+        while(tmp < ScaleEPS && tmp > 0.0){
            for(j = 0; j < *nc; j++) X[i + j* *nr] *=ScaleMAX;
            result[i] +=1L;
            tmp *= ScaleMAX;
@@ -245,7 +248,7 @@ void lll3(SEXP dlist, double *eva, double *eve, double *evei, double *el, double
 SEXP PML0(SEXP dlist, SEXP EL, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP nco, SEXP contrast, SEXP N){
     int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, indLL;
     int nTips = INTEGER(NTips)[0], *SC;
-    double *g=REAL(G), *tmp, logScaleEPS;
+    double *g=REAL(G), *tmp; //, logScaleEPS;
     SEXP TMP;
     double *eva, *eve, *evei;
     eva = REAL(VECTOR_ELT(eig, 0));
@@ -260,8 +263,8 @@ SEXP PML0(SEXP dlist, SEXP EL, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP
     for(i=0; i<k; i++){
         lll(dlist, eva, eve, evei, REAL(EL), g[i], &nr, &nc, INTEGER(node), INTEGER(edge), nTips, REAL(contrast), INTEGER(nco)[0], INTEGER(N)[0], &SC[nr * i], REAL(bf), &tmp[i*nr], &LL[indLL *i]);
     }
-    logScaleEPS = log(ScaleEPS);
-    for(i=0; i<(k*nr); i++) tmp[i] = logScaleEPS * SC[i] + log(tmp[i]);
+    // logScaleEPS = log(ScaleEPS);
+    for(i=0; i<(k*nr); i++) tmp[i] = LOG_SCALE_EPS * SC[i] + log(tmp[i]);
     UNPROTECT(1);
     return TMP;
 }
@@ -297,7 +300,8 @@ SEXP PML4(SEXP dlist, SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP ei
     return TMP;
 }
 
-
+//  LL /= (child P)
+//  child *= (LL *P)
 void moveLL5(double *LL, double *child, double *P, int *nr, int *nc, double *tmp){
     int j;
     F77_CALL(dgemm)("N", "N", nr, nc, nc, &one, child, nr, P, nc, &zero, tmp, nr FCONE FCONE);
@@ -321,19 +325,19 @@ void helpPrep(double *dad, double *child, double *eve, double *evi, int nr, int
     for(int j=0; j<(nc * nr); j++) res[j]*=tmp[j];
 }
 
-
+/*
 void helpDAD2(double *dad, int *child, double *contrast, double *P, int nr, int nc, int nco, double *res){
     matp(child, contrast, P, &nr, &nc, &nco, res);
     for(int j=0; j<(nc * nr); j++) res[j]=dad[j]/res[j];
 }
-
+*/
 
 void helpDAD5(double *dad, int *child, double *contrast, double *P, int nr, int nc, int nco, double *res){
     matp(child, contrast, P, &nr, &nc, &nco, res);
     for(int j=0; j<(nc * nr); j++) dad[j]/=res[j];
 }
 
-
+/*
 SEXP getDAD2(SEXP dad, SEXP child, SEXP contrast, SEXP P, SEXP nr, SEXP nc, SEXP nco){
     R_len_t i, n=length(P);
     int ncx=INTEGER(nc)[0], nrx=INTEGER(nr)[0], nrs=INTEGER(nco)[0]; //, j
@@ -348,7 +352,7 @@ SEXP getDAD2(SEXP dad, SEXP child, SEXP contrast, SEXP P, SEXP nr, SEXP nc, SEXP
     UNPROTECT(1); //RESULT
     return(RESULT);
 }
-
+*/
 
 void helpPrep2(double *dad, int *child, double *contrast, double *evi, int nr, int nc, int nrs, double *res){
     int i, j;
@@ -358,7 +362,7 @@ void helpPrep2(double *dad, int *child, double *contrast, double *evi, int nr, i
     }
 }
 
-
+/*
 SEXP getPrep2(SEXP dad, SEXP child, SEXP contrast, SEXP evi, SEXP nr, SEXP nc, SEXP nco){
     R_len_t i, n=length(dad);
     int ncx=INTEGER(nc)[0], nrx=INTEGER(nr)[0], ncs=INTEGER(nco)[0];
@@ -373,7 +377,7 @@ SEXP getPrep2(SEXP dad, SEXP child, SEXP contrast, SEXP evi, SEXP nr, SEXP nc, S
     UNPROTECT(1);
     return(RESULT);
 }
-
+*/
 
 // child *= (dad * P)
 void goDown(double *dad, double *child, double *P, int nr, int nc, double *res){
@@ -573,12 +577,6 @@ SEXP LogLik2(SEXP dlist, SEXP P, SEXP nr, SEXP nc, SEXP node, SEXP edge, SEXP nT
     return(ans);
 }
 
-// raus
-//void matprod(double *x, int nrx, int ncx, double *y, int nry, int ncy, double *z)
-//{
-//    F77_CALL(dgemm)("N", "N", &nrx, &ncy, &ncx, &one, x, &nrx, y, &nry, &zero, z, &nrx FCONE FCONE);
-//}
-
 
 SEXP getM3(SEXP dad, SEXP child, SEXP P, SEXP nr, SEXP nc){
     R_len_t i, n=length(P);
@@ -617,7 +615,7 @@ void fs3(double *eva, int nc, double el, double *w, double *g, double *X, int ld
     for(i=0; i<nr; i++)f[i] = f0[i];
     NR_f(eva, nc, edle, w, g, X, ld, nr, f); // nc-1L !!!
     if(mkv){
-      for(j=0; j<nc; j++)p0 += f[i];
+      for(j=0; j<nr; j++)p0 += f[i];
       for(i=0; i<nr ;i++) f[i] /= (1-p0);
     }
     for(i=0; i<nr ;i++){
@@ -746,7 +744,6 @@ SEXP optE(SEXP PARENT, SEXP CHILD, SEXP ANC, SEXP eig, SEXP EVI, SEXP EL,
     evei = REAL(VECTOR_ELT(eig, 2));
 
     loli = parent[0];
-
     for(m = 0; m < n; m++){
         pa = parent[m];
         ch = child[m];
